@@ -1,5 +1,5 @@
 /* uuencode utility.
-   Copyright (C) 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1994, 1995, 1996, 2002 Free Software Foundation, Inc.
 
    This product is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,6 +50,17 @@
 /* Reworked to GNU style by Ian Lance Taylor, ian@airs.com, August 93.  */
 
 #include "system.h"
+#include "basename.h"
+#include "error.h"
+#include "exit.h"
+
+#if HAVE_LOCALE_H
+# include <locale.h>
+#else
+# define setlocale(Category, Locale)
+#endif
+#include "gettext.h"
+#define _(str) gettext (str)
 
 /*=======================================================\
 | uuencode [INPUT] OUTPUT				 |
@@ -69,8 +80,13 @@ static struct option longopts[] =
   { NULL, 0, 0, 0 }
 };
 
+static inline void try_putchar __P ((int));
 static void encode __P ((void));
-static void usage __P ((int));
+static void usage __P ((int))
+#if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
+     __attribute__ ((noreturn))
+#endif
+;
 
 /* The name this program was run with. */
 const char *program_name;
@@ -107,6 +123,14 @@ const char uu_base64[64] =
 /* ENC is the basic 1 character encoding function to make a char printing.  */
 #define ENC(Char) (trans_ptr[(Char) & 077])
 
+static inline void
+try_putchar (c)
+         int c;
+{
+  if (putchar (c) == EOF)
+    error (EXIT_FAILURE, 0, _("Write error"));
+}
+
 /*------------------------------------------------.
 | Copy from IN to OUT, encoding as you go along.  |
 `------------------------------------------------*/
@@ -114,92 +138,57 @@ const char uu_base64[64] =
 static void
 encode ()
 {
-  register int ch, n;
+  register int n;
+  int finishing = 0;
   register char *p;
-  char buf[80];
+  char buf[46];  /* 45 should be enough, but one never knows... */
 
-  while (1)
+  while ( !finishing && (n = fread (buf, 1, 45, stdin)) > 0 )
     {
-      n = 0;
-      do
-	{
-	  register int m = fread (buf, 1, 45 - n, stdin);
-	  if (m == 0)
-	    break;
-	  n += m;
-	}
-      while (n < 45);
-
-      if (n == 0)
-	break;
+      if (n < 45)
+        {
+          if (feof (stdin))
+            finishing = 1;
+          else
+            error (EXIT_FAILURE, 0, _("Read error"));
+        }
 
       if (trans_ptr == uu_std)
-	if (putchar (ENC (n)) == EOF)
-	  break;
+        putchar (ENC (n));
+
       for (p = buf; n > 2; n -= 3, p += 3)
-	{
-	  ch = *p >> 2;
-	  ch = ENC (ch);
-	  if (putchar (ch) == EOF)
-	    break;
-	  ch = ((*p << 4) & 060) | ((p[1] >> 4) & 017);
-	  ch = ENC (ch);
-	  if (putchar (ch) == EOF)
-	    break;
-	  ch = ((p[1] << 2) & 074) | ((p[2] >> 6) & 03);
-	  ch = ENC (ch);
-	  if (putchar (ch) == EOF)
-	    break;
-	  ch = p[2] & 077;
-	  ch = ENC (ch);
-	  if (putchar (ch) == EOF)
-	    break;
-	}
+        {
+          try_putchar (ENC (*p >> 2));
+          try_putchar (ENC (((*p << 4) & 060) | ((p[1] >> 4) & 017)));
+          try_putchar (ENC (((p[1] << 2) & 074) | ((p[2] >> 6) & 03)));
+          try_putchar (ENC (p[2] & 077));
+        }
 
-      if (n != 0)
-	break;
+      if (n > 0)  /* encode the last one or two chars */
+        {
+          char tail = trans_ptr == uu_std ? ENC ('\0') : '=';
 
-      if (putchar ('\n') == EOF)
-	break;
-    }
+          if (n == 1)
+            p[1] = '\0';
 
-  while (n != 0)
-    {
-      char c1 = *p;
-      char c2 = n == 1 ? 0 : p[1];
+          try_putchar (ENC (*p >> 2));
+          try_putchar (ENC (((*p << 4) & 060) | ((p[1] >> 4) & 017)));
+          try_putchar (n == 1 ? tail : ENC ((p[1] << 2) & 074));
+          try_putchar (tail);
+        }
 
-      ch = c1 >> 2;
-      ch = ENC (ch);
-      if (putchar (ch) == EOF)
-	break;
-
-      ch = ((c1 << 4) & 060) | ((c2 >> 4) & 017);
-      ch = ENC (ch);
-      if (putchar (ch) == EOF)
-	break;
-
-      if (n == 1)
-	ch = trans_ptr == uu_std ? ENC ('\0') : '=';
-      else
-	{
-	  ch = (c2 << 2) & 074;
-	  ch = ENC (ch);
-	}
-      if (putchar (ch) == EOF)
-	break;
-      ch = trans_ptr == uu_std ? ENC ('\0') : '=';
-      if (putchar (ch) == EOF)
-	break;
-      putchar ('\n');
-      break;
+      try_putchar ('\n');
     }
 
   if (ferror (stdin))
     error (EXIT_FAILURE, 0, _("Read error"));
+  if (fclose (stdin) != 0)
+    error (EXIT_FAILURE, errno, _("Read error"));
+
   if (trans_ptr == uu_std)
     {
-      putchar (ENC ('\0'));
-      putchar ('\n');
+      try_putchar (ENC ('\0'));
+      try_putchar ('\n');
     }
 }
 
@@ -213,10 +202,12 @@ usage (status)
   else
     {
       printf (_("Usage: %s [INFILE] REMOTEFILE\n"), program_name);
-      printf (_("\n\
-  -h, --help      display this help and exit\n\
+      fputs (_("\n\
   -m, --base64    use base64 encoding as of RFC1521\n\
-  -v, --version   output version information and exit\n"));
+      --help      display this help and exit\n\
+      --version   output version information and exit\n"), stdout);
+      /* TRANSLATORS: add the contact address for your translation team! */
+      printf (_("Report bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -240,7 +231,7 @@ main (argc, argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while (opt = getopt_long (argc, argv, "hmv", longopts, (int *) NULL),
+  while (opt = getopt_long (argc, argv, "hm", longopts, (int *) NULL),
 	 opt != EOF)
     {
       switch (opt)
@@ -253,7 +244,14 @@ main (argc, argv)
 	  break;
 
 	case 'v':
-	  printf ("%s - GNU %s %s\n", program_name, PACKAGE, VERSION);
+	  printf ("%s (GNU %s) %s\n", basename (program_name),
+		  PACKAGE, VERSION);
+	  /* xgettext: no-wrap */
+	  printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+"),
+		  "1994, 1995, 1996, 2002");
 	  exit (EXIT_SUCCESS);
 
 	case 0:
@@ -289,11 +287,16 @@ main (argc, argv)
 choke me - Must translate mode argument
 #endif
 
-  printf ("begin%s %o %s\n", trans_ptr == uu_std ? "" : "-base64",
-	  mode, argv[optind]);
+  if (printf ("begin%s %o %s\n", trans_ptr == uu_std ? "" : "-base64",
+	      mode, argv[optind]) < 0)
+    error (EXIT_FAILURE, errno, _("Write error"));
+
   encode ();
-  printf (trans_ptr == uu_std ? "end\n" : "====\n");
-  if (ferror (stdout))
-    error (EXIT_FAILURE, 0, _("Write error"));
+
+  if (ferror (stdout) ||
+      printf (trans_ptr == uu_std ? "end\n" : "====\n") < 0 ||
+      fclose (stdout) != 0)
+    error (EXIT_FAILURE, errno, _("Write error"));
+
   exit (EXIT_SUCCESS);
 }
